#!/usr/bin/env python3
"""
Claude PM Framework - Python Memory Monitor
Replaces the JavaScript memory monitor with a pure Python implementation.
"""

import asyncio
import os
import psutil
import time
import json
import logging
from typing import Dict, Optional, Set
from datetime import datetime
from pathlib import Path

logger = logging.getLogger(__name__)


class MemoryMonitor:
    """Pure Python memory monitoring for the Claude PM framework."""
    
    def __init__(self, log_dir: Optional[Path] = None):
        self.config = {
            'memory_threshold_percent': 80,  # Alert when memory > 80%
            'check_interval': 5,  # Check every 5 seconds
            'subprocess_memory_limit_mb': 1500,  # 1.5GB per subprocess
            'max_subprocesses': 5,  # Maximum concurrent subprocesses
        }
        
        self.log_dir = log_dir or Path('.claude-pm/logs')
        self.log_dir.mkdir(parents=True, exist_ok=True)
        self.log_file = self.log_dir / 'memory-monitor.log'
        
        self._running = False
        self._tracked_pids: Set[int] = set()
        
    def get_memory_status(self) -> Dict:
        """Get current memory status of the system and process."""
        # System memory
        vm = psutil.virtual_memory()
        
        # Current process memory
        process = psutil.Process()
        process_info = process.memory_info()
        
        # Get child processes
        children = []
        try:
            for child in process.children(recursive=True):
                try:
                    child_info = child.memory_info()
                    children.append({
                        'pid': child.pid,
                        'name': child.name(),
                        'memory_mb': child_info.rss / 1024 / 1024,
                        'status': child.status()
                    })
                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    pass
        except psutil.NoSuchProcess:
            pass
        
        return {
            'timestamp': datetime.now().isoformat(),
            'system': {
                'total_mb': vm.total / 1024 / 1024,
                'available_mb': vm.available / 1024 / 1024,
                'percent': vm.percent,
                'used_mb': vm.used / 1024 / 1024
            },
            'process': {
                'pid': process.pid,
                'memory_mb': process_info.rss / 1024 / 1024,
                'cpu_percent': process.cpu_percent(interval=0.1)
            },
            'children': children,
            'child_count': len(children),
            'total_child_memory_mb': sum(c['memory_mb'] for c in children)
        }
    
    def check_memory_limits(self, status: Dict) -> Dict:
        """Check if memory usage exceeds configured limits."""
        alerts = []
        
        # System memory check
        if status['system']['percent'] > self.config['memory_threshold_percent']:
            alerts.append({
                'level': 'WARNING',
                'message': f"System memory usage is {status['system']['percent']:.1f}%"
            })
        
        # Subprocess count check
        if status['child_count'] > self.config['max_subprocesses']:
            alerts.append({
                'level': 'WARNING',
                'message': f"Too many subprocesses: {status['child_count']} (max: {self.config['max_subprocesses']})"
            })
        
        # Individual subprocess memory check
        for child in status['children']:
            if child['memory_mb'] > self.config['subprocess_memory_limit_mb']:
                alerts.append({
                    'level': 'WARNING',
                    'message': f"Subprocess {child['pid']} ({child['name']}) using {child['memory_mb']:.1f}MB"
                })
        
        return alerts
    
    def cleanup_zombies(self) -> int:
        """Clean up zombie processes."""
        cleaned = 0
        for proc in psutil.process_iter(['pid', 'status', 'ppid']):
            try:
                if proc.info['status'] == psutil.STATUS_ZOMBIE:
                    # Only clean zombies that are our children
                    if proc.info['ppid'] == os.getpid():
                        os.waitpid(proc.info['pid'], os.WNOHANG)
                        cleaned += 1
                        logger.info(f"Cleaned zombie process {proc.info['pid']}")
            except (psutil.NoSuchProcess, OSError):
                pass
        return cleaned
    
    def log_status(self, status: Dict, alerts: list):
        """Log memory status to file."""
        log_entry = {
            'timestamp': status['timestamp'],
            'level': 'ERROR' if any(a['level'] == 'ERROR' for a in alerts) else 
                     'WARNING' if alerts else 'INFO',
            'status': status,
            'alerts': alerts
        }
        
        with open(self.log_file, 'a') as f:
            f.write(json.dumps(log_entry) + '\n')
    
    async def monitor_loop(self):
        """Main monitoring loop."""
        logger.info("Starting memory monitor")
        self._running = True
        
        while self._running:
            try:
                # Get memory status
                status = self.get_memory_status()
                
                # Check limits
                alerts = self.check_memory_limits(status)
                
                # Clean zombies periodically
                if int(time.time()) % 30 == 0:  # Every 30 seconds
                    zombies_cleaned = self.cleanup_zombies()
                    if zombies_cleaned > 0:
                        logger.info(f"Cleaned {zombies_cleaned} zombie processes")
                
                # Log status
                self.log_status(status, alerts)
                
                # Print alerts to console
                for alert in alerts:
                    logger.warning(f"{alert['level']}: {alert['message']}")
                
            except Exception as e:
                logger.error(f"Error in memory monitor: {e}")
            
            await asyncio.sleep(self.config['check_interval'])
    
    def start(self):
        """Start the memory monitor."""
        asyncio.create_task(self.monitor_loop())
    
    def stop(self):
        """Stop the memory monitor."""
        self._running = False
        logger.info("Memory monitor stopped")


# Singleton instance
_monitor_instance: Optional[MemoryMonitor] = None


def get_memory_monitor(log_dir: Optional[Path] = None) -> MemoryMonitor:
    """Get or create the singleton memory monitor instance."""
    global _monitor_instance
    if _monitor_instance is None:
        _monitor_instance = MemoryMonitor(log_dir)
    return _monitor_instance


if __name__ == "__main__":
    # Test the monitor
    import asyncio
    
    async def test_monitor():
        monitor = get_memory_monitor()
        await monitor.monitor_loop()
    
    try:
        asyncio.run(test_monitor())
    except KeyboardInterrupt:
        print("\nMonitor stopped")